@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -53,10 +53,13 @@ import static com.openexchange.sessiond.services.SessiondServiceRegistry.getServ
 import java.util.concurrent.locks.Lock;
 import java.util.concurrent.locks.ReadWriteLock;
 import java.util.concurrent.locks.ReentrantReadWriteLock;
+import org.apache.commons.logging.Log;
+import org.apache.commons.logging.LogFactory;
 import com.openexchange.caching.Cache;
 import com.openexchange.caching.CacheException;
 import com.openexchange.caching.CacheKey;
 import com.openexchange.caching.CacheService;
+import com.openexchange.caching.InvalidatedMarker;
 import com.openexchange.caching.objects.CachedSession;
 import com.openexchange.server.ServiceException;
 
@@ -65,11 +68,13 @@ import com.openexchange.server.ServiceException;
  * <p>
  * <b>Note</b>: The appropriate instance of {@link CacheService} is obtained on every request thus there's no need to to release/re-init any
  * references.
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public final class SessionCache {
 
+    private static final Log LOG = com.openexchange.log.Log.valueOf(LogFactory.getLog(SessionCache.class));
+
     static final String LATERAL_REGION_NAME = "SessionLTCP";
 
     static final String REGION_NAME = "SessionCache";
@@ -83,7 +88,7 @@ public final class SessionCache {
         readWriteLock = new ReentrantReadWriteLock();
         /**
          * Uncomment this to enable default element event handler
-         * 
+         *
          * <pre>
          * final Cache cache;
          * try {
@@ -99,7 +104,14 @@ public final class SessionCache {
          */
     }
 
-    private Cache getCache() throws ServiceException, CacheException {
+    /**
+     * Gets associated cache.
+     * 
+     * @return The cache
+     * @throws ServiceException If cache is absent
+     * @throws CacheException If cache cannot be returned
+     */
+    public Cache getCache() throws ServiceException, CacheException {
         final CacheService cacheService = getServiceRegistry().getService(CacheService.class);
         if (null == cacheService) {
             throw new ServiceException(ServiceException.Code.SERVICE_UNAVAILABLE, CacheService.class.getName());
@@ -131,9 +143,49 @@ public final class SessionCache {
         }
     }
 
+    /**
+     * Checks if cache contains invalidate marker for given context (and performs local-remove it if present).
+     *
+     * @param contextId The context identifier
+     * @return <code>true</code> if present in cache; otherwise <code>false</code>
+     * @throws CacheException If a caching error occurs
+     * @throws ServiceException If caching service is not available
+     */
+    @SuppressWarnings("unchecked")
+    public boolean containsInvalidateMarker(final int contextId) throws ServiceException, CacheException {
+        final Cache cache = getCache();
+        final Lock readLock = readWriteLock.readLock();
+        readLock.lock();
+        try {
+            final Integer key = Integer.valueOf(contextId);
+            Object element = cache.get(key);
+            if ((null != element) && (element instanceof InvalidatedMarker) && (contextId == ((InvalidatedMarker<Integer>) element).getIdentifier().intValue())) {
+                final Lock writeLock = readWriteLock.writeLock();
+                readLock.unlock();
+                writeLock.lock();
+                try {
+                    element = cache.get(key);
+                    if (null != element) {
+                        cache.localRemove(key);
+                        if (LOG.isInfoEnabled()) {
+                            LOG.info("Detected & (locally) removed invalidate marker for context: " + contextId);
+                        }
+                    }
+                } finally {
+                    readLock.lock();
+                    writeLock.unlock();
+                }
+                return true;
+            }
+            return false;
+        } finally {
+            readLock.unlock();
+        }
+    }
+
     /**
      * Removes and returns a cached session from cache
-     * 
+     *
      * @param sessionId The session identifier (which is sent as <i>"session=..."</i> in every request)
      * @param contextId The context ID
      * @return A cached session or <code>null</code>
@@ -188,7 +240,7 @@ public final class SessionCache {
      * Puts given cache-able session into cache if none user-bound session is already contained in cache.
      * <p>
      * The secret cookie identifier obtained by {@link CachedSession#getSecret()} is used as key.
-     * 
+     *
      * @param cachedSession The cache-able session to put into cache
      * @return <code>true</code> if cache-able session could be successfully cached; otherwise <code>false</code>
      * @throws CacheException If put into cache fails
@@ -240,7 +292,7 @@ public final class SessionCache {
 
     /**
      * Puts given cache-able session into cache to distribute a remove for associated session among auxiliary caches.
-     * 
+     *
      * @param cachedSession The cached session which shall be removed in auxiliary caches
      * @throws CacheException If put into cache fails
      * @throws ServiceException If caching service is not available
@@ -266,7 +318,7 @@ public final class SessionCache {
 
     /**
      * Checks if cache already holds a user-bound cached session
-     * 
+     *
      * @param sessionId The secret cookie identifier (which is sent as <i>"session=..."</i> in every request)
      * @return <code>true</code> if a user-bound cached session is already present in cache; otherwise <code>false</code>
      * @throws CacheException If a caching error occurs
@@ -285,7 +337,7 @@ public final class SessionCache {
 
     /**
      * Gets the first encountered cached session for given user in specified context
-     * 
+     *
      * @param userId The user ID
      * @param contextId The context ID
      * @return The first encountered cached session for given user in specified context or <code>null</code> if none found
@@ -309,7 +361,7 @@ public final class SessionCache {
 
     /**
      * This method removes the dummy integer from the session cache. This triggers a remote remove and a broken connection is detected.
-     * 
+     *
      * @throws CacheException
      */
     public void testConnection() throws CacheException {
