@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -51,29 +51,24 @@ package com.openexchange.sessiond.cache;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
+import com.openexchange.timer.ScheduledTimerTask;
+import com.openexchange.timer.TimerService;
 
 /**
  * This timer tests the outgoing connections of the session cache.
- * 
+ *
  * @author <a href="mailto:marcus@open-xchange.org">Marcus Klein</a>
  */
 public class SessionCacheTimer implements Runnable {
 
-    /**
-     * Logger.
-     */
-    private static final Log LOG = LogFactory.getLog(SessionCacheTimer.class);
+    private static final Log LOG = com.openexchange.log.Log.valueOf(LogFactory.getLog(SessionCacheTimer.class));
+
+    private static ScheduledTimerTask sessionCacheTimer;
 
-    /**
-     * Default constructor.
-     */
     public SessionCacheTimer() {
         super();
     }
 
-    /**
-     * {@inheritDoc}
-     */
     public void run() {
         try {
             SessionCache.getInstance().testConnection();
@@ -81,4 +76,15 @@ public class SessionCacheTimer implements Runnable {
             LOG.error(t.getMessage(), t);
         }
     }
+
+    public static void addTimerService(TimerService service) {
+        sessionCacheTimer = service.scheduleWithFixedDelay(new SessionCacheTimer(), 0, 30000);
+    }
+
+    public static void removeTimerService() {
+        if (null != sessionCacheTimer) {
+            sessionCacheTimer.cancel(false);
+            sessionCacheTimer = null;
+        }
+    }
 }
