@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -51,34 +51,49 @@ package com.openexchange.sessiond.event;
 
 import java.util.Map;
 import com.openexchange.session.Session;
-import com.openexchange.sessiond.exception.SessiondException;
+import com.openexchange.sessiond.SessiondException;
 
 /**
  * {@link SessiondEventListener} - The listener for sessiond events.
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public interface SessiondEventListener {
 
     /**
      * Handles the removal of specified session
-     * 
+     *
      * @param session The removed session
      */
-    public void handleSessionRemoval(Session session);
+    void handleSessionRemoval(Session session);
 
     /**
      * Handles the removal of specified session container
-     * 
+     *
      * @param sessions The removed session container
      */
-    public void handleContainerRemoval(Map<String, Session> sessions);
+    void handleContainerRemoval(Map<String, Session> sessions);
 
     /**
      * Handles the specified error
-     * 
+     *
      * @param error The error
      */
     public void handleError(SessiondException error);
 
+    /**
+     * Implementations should remove all temporary data for those sessions because they are not used for a longer time frame and are now
+     * stored in the long term session life time container. This should use as less memory as possible.
+     *
+     * @param sessions the sessions put into long term container.
+     */
+    void handleSessionDataRemoval(Map<String, Session> sessions);
+
+    /**
+     * Implementations can restore temporary session information on this event. It is emitted if a session walks from the long term life
+     * time container back to the normal ones.
+     *
+     * @param session reactivated sessions.
+     */
+    void handleSessionReactivation(Session session);
 }
