@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -53,12 +53,13 @@ import java.security.MessageDigest;
 import java.security.NoSuchAlgorithmException;
 import java.security.SecureRandom;
 import java.util.UUID;
-import com.openexchange.sessiond.exception.SessiondException;
+import com.openexchange.sessiond.SessionExceptionCodes;
+import com.openexchange.sessiond.SessiondException;
 
 /**
  * {@link DefaultSessionIdGenerator} - The default session ID generator.
- * 
- * @author <a href="mailto:sebastian.kauss@netline-is.de">Sebastian Kauss</a>
+ *
+ * @author <a href="mailto:sebastian.kauss@open-xchange.com">Sebastian Kauss</a>
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public final class DefaultSessionIdGenerator extends SessionIdGenerator {
@@ -74,7 +75,7 @@ public final class DefaultSessionIdGenerator extends SessionIdGenerator {
     }
 
     @Override
-    public String createRandomId() throws SessiondException {
+    public String createRandomId() {
         return UUID.randomUUID().toString();
     }
 
@@ -93,17 +94,16 @@ public final class DefaultSessionIdGenerator extends SessionIdGenerator {
                 algorithm.update(buf);
                 digest = algorithm.digest();
             }
-            for (int i = 0; i < digest.length; i++) {
-                final String hex = Integer.toHexString(digest[i] & 0xff);
+            for (final byte element : digest) {
+                final String hex = Integer.toHexString(element & 0xff);
                 if (hex.length() < 2) {
                     builder.append('0');
                 }
                 builder.append(hex);
             }
             return builder.toString();
-        } catch (final NoSuchAlgorithmException exc) {
-            throw new SessiondException(SessiondException.Code.SESSIOND_EXCEPTION, exc);
+        } catch (final NoSuchAlgorithmException e) {
+            throw SessionExceptionCodes.SESSIOND_EXCEPTION.create(e);
         }
     }
-
 }
