@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,149 +49,202 @@
 
 package com.openexchange.sessiond.impl;
 
+import gnu.trove.ConcurrentTIntObjectHashMap;
+import gnu.trove.procedure.TObjectProcedure;
 import java.util.ArrayList;
 import java.util.Collection;
-import java.util.HashMap;
-import java.util.HashSet;
-import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.concurrent.ConcurrentHashMap;
-import java.util.concurrent.ConcurrentMap;
+import java.util.concurrent.locks.Lock;
+import java.util.concurrent.locks.ReentrantLock;
 import com.openexchange.session.Session;
+import com.openexchange.sessiond.SessionExceptionCodes;
+import com.openexchange.sessiond.SessiondException;
 
 /**
  * {@link SessionContainer} - A thread-safe container for {@link Session} objects wrapped by a {@link SessionControl} object.
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 final class SessionContainer {
 
-    private static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(SessionContainer.class);
-
-    private static final boolean DEBUG_ENABLED = LOG.isDebugEnabled();
-
     private static final Object PRESENT = new Object();
 
-    private final ConcurrentMap<String, SessionControl> sessionIdMap;
+    private final SessionMap sessionMap;
 
-    private final ConcurrentMap<UserKey, Map<String, Object>> userSessions;
+    private final Lock sessionIdMapLock = new ReentrantLock();
 
-    /**
-     * Initializes a new {@link SessionContainer session container}.
-     * 
-     * @param maxSessions The max. number of sessions kept by this container
-     */
-    SessionContainer(final int maxSessions) {
+    private final ConcurrentTIntObjectHashMap<ConcurrentTIntObjectHashMap<Map<String, Object>>> userSessions;
+
+    SessionContainer() {
         super();
-        sessionIdMap = new ConcurrentHashMap<String, SessionControl>(maxSessions);
-        userSessions = new ConcurrentHashMap<UserKey, Map<String, Object>>(maxSessions);
+        sessionMap = new SessionMap();
+        userSessions = new ConcurrentTIntObjectHashMap<ConcurrentTIntObjectHashMap<Map<String, Object>>>(32);
     }
 
     /**
      * Gets the current number of sessions held by this container
-     * 
+     *
      * @return The current number of sessions held by this container
      */
     int size() {
-        return sessionIdMap.size();
+        return sessionMap.size();
     }
 
     /**
      * Checks if this container contains an entry for specified session ID
-     * 
+     *
      * @param sessionId The session ID
      * @return <code>true</code> if this container contains an entry for specified session ID; otherwise <code>false</code>
      */
     boolean containsSessionId(final String sessionId) {
-        return sessionIdMap.containsKey(sessionId);
+        return sessionMap.containsBySessionId(sessionId);
+    }
+
+    /**
+     * Checks if this container contains an entry for specified alternative identifier.
+     *
+     * @param altId The alternative identifier
+     * @return <code>true</code> if this container contains an entry for specified alternative identifier; otherwise <code>false</code>
+     */
+    boolean containsAlternativeId(final String altId) {
+        return sessionMap.containsByAlternativeId(altId);
     }
 
     /**
      * Checks if this container contains a session for specified user in specified context
-     * 
+     *
      * @param userId The user ID
      * @param contextId The context ID
      * @return <code>true</code> if this container contains an entry for specified user; otherwise <code>false</code>
      */
     boolean containsUser(final int userId, final int contextId) {
-        return userSessions.containsKey(new UserKey(userId, contextId));
+        final ConcurrentTIntObjectHashMap<Map<String, Object>> map = userSessions.get(contextId);
+        return null != map && map.contains(userId);
     }
 
     /**
      * Gets the number of sessions bound to specified user in specified context
-     * 
+     *
      * @param userId The user ID
      * @param contextId The context ID
      * @return The number of sessions bound to specified user in specified context
      */
     int numOfUserSessions(final int userId, final int contextId) {
-        final Map<String, Object> sessionIds = userSessions.get(new UserKey(userId, contextId));
+        final ConcurrentTIntObjectHashMap<Map<String, Object>> map = userSessions.get(contextId);
+        if (null == map) {
+            return 0;
+        }
+        final Map<String, Object> sessionIds = map.get(userId);
         return null == sessionIds ? 0 : sessionIds.size();
     }
 
     /**
      * Gets the session bound to specified session ID.
-     * 
+     *
      * @param sessionId The session ID
      * @return The session bound to specified session ID, or <code>null</code> if there's no session for specified session ID.
      */
     SessionControl getSessionById(final String sessionId) {
-        return sessionIdMap.get(sessionId);
+        return sessionMap.getBySessionId(sessionId);
+    }
+
+    /**
+     * Gets the session bound to specified alternative identifier.
+     *
+     * @param altId The alternative identifier
+     * @return The session bound to specified alternative identifier, or <code>null</code> if there's no session for specified alternative identifier.
+     */
+    SessionControl getSessionByAlternativeId(final String altId) {
+        return sessionMap.getByAlternativeId(altId);
     }
 
     /**
      * Gets the sessions bound to specified user ID and context ID.
-     * 
+     *
      * @param userId The user ID
      * @param contextId The context ID
      * @return The sessions bound to specified user ID and context ID
      */
     SessionControl[] getSessionsByUser(final int userId, final int contextId) {
-        final Map<String, Object> sessionIds = userSessions.get(new UserKey(userId, contextId));
+        final ConcurrentTIntObjectHashMap<Map<String, Object>> map = userSessions.get(contextId);
+        if (null == map) {
+            return new SessionControl[0];
+        }
+        final Map<String, Object> sessionIds = map.get(userId);
         if (null == sessionIds) {
             return new SessionControl[0];
         }
         final List<SessionControl> l = new ArrayList<SessionControl>(sessionIds.size());
         for (final String sessionId : sessionIds.keySet()) {
-            l.add(sessionIdMap.get(sessionId));
+            l.add(sessionMap.getBySessionId(sessionId));
         }
         return l.toArray(new SessionControl[sessionIds.size()]);
     }
 
+    public SessionControl getAnySessionByUser(final int userId, final int contextId) {
+        final ConcurrentTIntObjectHashMap<Map<String, Object>> map = userSessions.get(contextId);
+        if (null == map) {
+            return null;
+        }
+        final Map<String, Object> sessionIds = map.get(userId);
+        if (sessionIds == null) {
+            return null;
+        }
+        for (final String sessionId : sessionIds.keySet()) {
+            return sessionMap.getBySessionId(sessionId);
+        }
+        return null;
+    }
+
+
     /**
      * Wraps specified session by a newly created {@link SessionControl} object and puts it into this container
-     * 
+     *
      * @param session The session to put
-     * @param timeToLive The session's time to live
      * @return The wrapping {@link SessionControl session control}.
      */
-    SessionControl put(final Session session, final int timeToLive) {
+    SessionControl put(final SessionImpl session) throws SessiondException {
         final String sessionId = session.getSessionID();
-        /*
-         * Add session
-         */
-        SessionControl sessionControl = sessionIdMap.get(sessionId);
-        if (null == sessionControl) {
-            final SessionControl newSessionControl = new SessionControl(session, timeToLive);
-            sessionControl = sessionIdMap.putIfAbsent(sessionId, newSessionControl);
+        // Add session
+        SessionControl sessionControl;
+        sessionIdMapLock.lock();
+        try {
+            sessionControl = sessionMap.getBySessionId(sessionId);
             if (null == sessionControl) {
-                sessionControl = newSessionControl;
-            } else {
-                sessionControl.renew(session, timeToLive);
-                if (DEBUG_ENABLED) {
-                    LOG.debug(new StringBuilder("session REBORN sessionid=").append(sessionId));
+                final SessionControl newSessionControl = new SessionControl(session);
+                sessionControl = sessionMap.putIfAbsentBySessionId(sessionId, newSessionControl);
+                if (null == sessionControl) {
+                    sessionControl = newSessionControl;
+                } else {
+                    final String login1 = sessionControl.getSession().getLogin();
+                    final String login2 = session.getLogin();
+                    throw SessionExceptionCodes.SESSIONID_COLLISION.create(login1, login2);
                 }
+            } else {
+                final String login1 = sessionControl.getSession().getLogin();
+                final String login2 = session.getLogin();
+                throw SessionExceptionCodes.SESSIONID_COLLISION.create(login1, login2);
+            }
+        } finally {
+            sessionIdMapLock.unlock();
+        }
+        // Add session ID to user-sessions-map
+        final int contextId = session.getContextId();
+        ConcurrentTIntObjectHashMap<Map<String, Object>> map = userSessions.get(contextId);
+        if (null == map) {
+            final ConcurrentTIntObjectHashMap<Map<String, Object>> newMap = new ConcurrentTIntObjectHashMap<Map<String,Object>>(32);
+            map = userSessions.putIfAbsent(contextId, newMap);
+            if (null == map) {
+                map = newMap;
             }
         }
-        /*
-         * Add session ID to user-sessions-map
-         */
-        final UserKey key = new UserKey(session.getUserId(), session.getContextId());
-        Map<String, Object> sessionIds = userSessions.get(key);
+        final int userId = session.getUserId();
+        Map<String, Object> sessionIds = map.get(userId);
         if (sessionIds == null) {
             final Map<String, Object> newSet = new ConcurrentHashMap<String, Object>();
-            sessionIds = userSessions.putIfAbsent(key, newSet);
+            sessionIds = map.putIfAbsent(userId, newSet);
             if (null == sessionIds) {
                 sessionIds = newSet;
             }
@@ -202,18 +255,34 @@ final class SessionContainer {
 
     /**
      * Puts specified {@link SessionControl} object into this container
-     * 
+     *
      * @param sessionControl The session control to put
+     * @throws SessiondException
      */
-    void putSessionControl(final SessionControl sessionControl) {
+    void putSessionControl(final SessionControl sessionControl) throws SessiondException {
         final Session session = sessionControl.getSession();
         final String sessionId = session.getSessionID();
-        sessionIdMap.put(sessionId, sessionControl);
-        final UserKey key = new UserKey(session.getUserId(), session.getContextId());
-        Map<String, Object> sessionIds = userSessions.get(key);
+        final SessionControl oldSessionControl = sessionMap.putIfAbsentBySessionId(sessionId, sessionControl);
+        if (null != oldSessionControl) {
+            final String login1 = oldSessionControl.getSession().getLogin();
+            final String login2 = sessionControl.getSession().getLogin();
+            throw SessionExceptionCodes.SESSIONID_COLLISION.create(login1, login2);
+        }
+        // Add session ID to user-sessions-map
+        final int contextId = session.getContextId();
+        ConcurrentTIntObjectHashMap<Map<String, Object>> map = userSessions.get(contextId);
+        if (null == map) {
+            final ConcurrentTIntObjectHashMap<Map<String, Object>> newMap = new ConcurrentTIntObjectHashMap<Map<String,Object>>(32);
+            map = userSessions.putIfAbsent(contextId, newMap);
+            if (null == map) {
+                map = newMap;
+            }
+        }
+        final int userId = session.getUserId();
+        Map<String, Object> sessionIds = map.get(userId);
         if (sessionIds == null) {
             final Map<String, Object> newSet = new ConcurrentHashMap<String, Object>();
-            sessionIds = userSessions.putIfAbsent(key, newSet);
+            sessionIds = map.putIfAbsent(userId, newSet);
             if (null == sessionIds) {
                 sessionIds = newSet;
             }
@@ -223,19 +292,52 @@ final class SessionContainer {
 
     /**
      * Removes the session bound to specified session ID.
-     * 
+     *
      * @param sessionId The session Id
      * @return The {@link SessionControl session control} previously associated with specified session ID, or <code>null</code>.
      */
     SessionControl removeSessionById(final String sessionId) {
-        final SessionControl sessionControl = sessionIdMap.remove(sessionId);
+        final SessionControl sessionControl = sessionMap.removeBySessionId(sessionId);
         if (sessionControl != null) {
             final Session session = sessionControl.getSession();
-            final UserKey key = new UserKey(session.getUserId(), session.getContextId());
-            final Map<String, Object> sessionIds = userSessions.get(key);
+            final int contextId = session.getContextId();
+            ConcurrentTIntObjectHashMap<Map<String, Object>> map = userSessions.get(contextId);
+            if (null == map) {
+                return sessionControl;
+            }
+            if (map.isEmpty()) {
+                final Lock l = userSessions.getReadWriteLock().writeLock();
+                l.lock();
+                try {
+                    map = userSessions.get(contextId);
+                    if (null == map) {
+                        return sessionControl;
+                    }
+                    if (map.isEmpty()) {
+                        userSessions.remove(contextId);
+                        return sessionControl;
+                    }
+                } finally {
+                    l.unlock();
+                }
+            }
+            final int userId = session.getUserId();
+            Map<String, Object> sessionIds = map.get(userId);
+            if (sessionIds == null) {
+                return sessionControl;
+            }
             sessionIds.remove(sessionId);
             if (sessionIds.isEmpty()) {
-                userSessions.remove(key);
+                final Lock l = map.getReadWriteLock().writeLock();
+                l.lock();
+                try {
+                    sessionIds = map.get(userId);
+                    if (null != sessionIds && sessionIds.isEmpty()) {
+                        map.remove(userId);
+                    }
+                } finally {
+                    l.unlock();
+                }
             }
         }
         return sessionControl;
@@ -243,20 +345,42 @@ final class SessionContainer {
 
     /**
      * Removes the sessions bound to specified user ID and context ID.
-     * 
+     *
      * @param userId The user ID
      * @param contextId The context ID
      * @return The {@link SessionControl session controls} previously associated with specified user ID and context ID.
      */
     SessionControl[] removeSessionsByUser(final int userId, final int contextId) {
-        final UserKey key = new UserKey(userId, contextId);
-        final Map<String, Object> sessionIds = userSessions.remove(key);
+        ConcurrentTIntObjectHashMap<Map<String, Object>> map = userSessions.get(contextId);
+        if (null == map) {
+            return new SessionControl[0];
+        }
+        if (map.isEmpty()) {
+            final Lock l = userSessions.getReadWriteLock().writeLock();
+            l.lock();
+            try {
+                map = userSessions.get(contextId);
+                if (null == map) {
+                    return new SessionControl[0];
+                }
+                if (map.isEmpty()) {
+                    userSessions.remove(contextId);
+                    return new SessionControl[0];
+                }
+            } finally {
+                l.unlock();
+            }
+        }
+        final Map<String, Object> sessionIds = map.remove(userId);
         if (sessionIds == null) {
             return new SessionControl[0];
         }
+        if (sessionIds.isEmpty()) {
+            return new SessionControl[0];
+        }
         final List<SessionControl> l = new ArrayList<SessionControl>(sessionIds.size());
         for (final String sessionId : sessionIds.keySet()) {
-            final SessionControl sc = sessionIdMap.remove(sessionId);
+            final SessionControl sc = sessionMap.removeBySessionId(sessionId);
             if (sc != null) {
                 l.add(sc);
             }
@@ -265,88 +389,50 @@ final class SessionContainer {
     }
 
     /**
-     * Converts this container to a newly created {@link Map map} holding this container's current sessions.
-     * <p>
-     * Changes made to any {@link Session} objects are reflected in this container.
-     * 
-     * @return The newly created {@link Map map} holding this container's sessions.
+     * Removes the sessions bound to specified context ID.
+     *
+     * @param contextId The context ID
+     * @return The {@link SessionControl session controls} previously associated with specified user ID and context ID.
      */
-    Map<String, Session> convert() {
-        final Map<String, Session> retval = new HashMap<String, Session>(sessionIdMap.size());
-        for (final Iterator<Map.Entry<String, SessionControl>> iterator = sessionIdMap.entrySet().iterator(); iterator.hasNext();) {
-            final Map.Entry<String, SessionControl> e = iterator.next();
-            retval.put(e.getKey(), e.getValue().getSession());
+    SessionControl[] removeSessionsByContext(final int contextId) {
+        final ConcurrentTIntObjectHashMap<Map<String, Object>> map = userSessions.remove(contextId);
+        if (null == map) {
+            return new SessionControl[0];
         }
-        return retval;
+        if (map.isEmpty()) {
+            return new SessionControl[0];
+        }
+        final List<SessionControl> l = new ArrayList<SessionControl>(128);
+        final SessionMap sessionMap = this.sessionMap;
+        map.forEachValue(new TObjectProcedure<Map<String, Object>>() {
+
+            public boolean execute(final Map<String, Object> sessionIds) {
+                for (final String sessionId : sessionIds.keySet()) {
+                    final SessionControl sc = sessionMap.removeBySessionId(sessionId);
+                    if (sc != null) {
+                        l.add(sc);
+                    }
+                }
+                return true;
+            }
+        });
+        return l.toArray(new SessionControl[l.size()]);
     }
 
-    /**
-     * Creates an {@link Iterator iterator} over the session IDs in this container.
-     * <p>
-     * The {@link Iterator#remove()} is not reflected in this container.
-     * 
-     * @return A newly created {@link Iterator iterator} over the session IDs in this container.
-     */
-    Iterator<String> getSessionIds() {
-        return new HashSet<String>(sessionIdMap.keySet()).iterator();
+    boolean hasForContext(final int contextId) {
+        final ConcurrentTIntObjectHashMap<Map<String, Object>> map = userSessions.get(contextId);
+        return null != map && !map.isEmpty();
     }
 
     /**
      * Returns a collection view of the {@link SessionControl} objects contained in this container. The collection is
      * <b><small>not</small></b> backed by the container, so changes to the map are not reflected in the container, but changes made to any
      * {@link SessionControl} object is reflected in this container.
-     * 
+     *
      * @return A collection view of the {@link SessionControl} objects contained in this container.
      */
     Collection<SessionControl> getSessionControls() {
-        return new ArrayList<SessionControl>(sessionIdMap.values());
+        return new ArrayList<SessionControl>(sessionMap.values());
     }
 
-    /**
-     * Simple key class for user ID and context ID.
-     */
-    private static final class UserKey {
-
-        private final int userId;
-
-        private final int cid;
-
-        private final int hash;
-
-        public UserKey(final int userId, final int cid) {
-            super();
-            this.userId = userId;
-            this.cid = cid;
-            final int prime = 31;
-            int result = 1;
-            result = prime * result + cid;
-            result = prime * result + userId;
-            hash = result;
-        }
-
-        @Override
-        public int hashCode() {
-            return hash;
-        }
-
-        @Override
-        public boolean equals(final Object obj) {
-            if (this == obj) {
-                return true;
-            }
-            if (!(obj instanceof UserKey)) {
-                return false;
-            }
-            final UserKey other = (UserKey) obj;
-            if (cid != other.cid) {
-                return false;
-            }
-            if (userId != other.userId) {
-                return false;
-            }
-            return true;
-        }
-
-    } // End of UserKey
-
 }
