@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,11 +49,12 @@
 
 package com.openexchange.sessiond.impl;
 
+import java.util.concurrent.atomic.AtomicLong;
 import com.openexchange.session.Session;
 
 /**
  * {@link SessionControl} - Holds a {@link Session} instance and remembers life-cycle timestamps such as last-accessed, creation-time, etc.
- * 
+ *
  * @author <a href="mailto:sebastian.kauss@open-xchange.com">Sebastian Kauss</a>
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
@@ -62,110 +63,64 @@ public class SessionControl {
     /**
      * Time stamp when this session control was created.
      */
-    private long creationTime;
-
-    /**
-     * The last-accessed time stamp.
-     */
-    private long lastAccessed;
+    private final long creationTime;
 
     /**
      * The associated session.
      */
-    private Session session;
+    private final SessionImpl session;
 
     /**
-     * The life time of this session control.
+     * Last-accessed time stamp.
      */
-    private int lifetime;
+    private final AtomicLong lastAccessed;
 
     /**
      * Initializes a new {@link SessionControl}
-     * 
+     *
      * @param session The stored session
-     * @param lifetime The session's life time
      */
-    public SessionControl(final Session session, final int lifetime) {
+    public SessionControl(final SessionImpl session) {
         super();
-        renew0(session, lifetime);
+        lastAccessed = new AtomicLong(System.currentTimeMillis());
+        this.session = session;
+        creationTime = System.currentTimeMillis();
     }
 
     /**
-     * (Atomically) Renews this session control:
-     * <ul>
-     * <li>Applies specified session</li>
-     * <li>Sets specified life time</li>
-     * <li>Sets creation time stamp to current time millis</li>
-     * <li>Sets last-accessed time stamp to current time millis</li>
-     * </ul>
+     * Gets the last-accessed time stamp.
      * 
-     * @param lifetime The (new) life time to set
+     * @return The last-accessed time stamp
      */
-    public void renew(final Session session, final int lifetime) {
-        synchronized (this) {
-            renew0(session, lifetime);
-        }
+    public long getLastAccessed() {
+        return lastAccessed.get();
     }
 
-    private void renew0(final Session session, final int lifetime) {
-        this.session = session;
-        this.lifetime = lifetime;
-        final long now = System.currentTimeMillis();
-        creationTime = now;
-        lastAccessed = now;
+    /**
+     * Updates last-accessed time stamp.
+     * 
+     * @return This control with last-accessed time stamp updated
+     */
+    public SessionControl touch() {
+        lastAccessed.set(System.currentTimeMillis());
+        return this;
     }
 
     /**
      * Gets the stored session
-     * 
+     *
      * @return The stored session
      */
-    public Session getSession() {
+    public SessionImpl getSession() {
         return session;
     }
 
     /**
      * Gets the creation-time timestamp
-     * 
+     *
      * @return The creation-time timestamp
      */
     public long getCreationTime() {
         return creationTime;
     }
-
-    /**
-     * Gets the last-accessed timestamp.
-     * 
-     * @return The last-accessed timestamp
-     */
-    public long getLastAccessed() {
-        return lastAccessed;
-    }
-
-    /**
-     * Gets the session's life time in milliseconds
-     * 
-     * @return The session's life time in milliseconds
-     */
-    public int getLifetime() {
-        return lifetime;
-    }
-
-    /**
-     * Updates session's last-accessed timestamp.
-     */
-    public void updateLastAccessed() {
-        lastAccessed = System.currentTimeMillis();
-    }
-
-    /**
-     * Checks if a session is still valid. Therefore the maximum lifetime of a session, a disabled context and a disabled user are checked.
-     * 
-     * @param session Session to check.
-     * @return <code>true</code> if the session is still valid.
-     */
-    public boolean isValid() {
-        return ((lastAccessed + lifetime) >= System.currentTimeMillis());
-    }
-
 }
