@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -63,21 +63,22 @@ import org.osgi.service.event.Event;
 import org.osgi.service.event.EventAdmin;
 import com.openexchange.caching.CacheException;
 import com.openexchange.caching.objects.CachedSession;
+import com.openexchange.groupware.AbstractOXException;
 import com.openexchange.groupware.contexts.Context;
 import com.openexchange.server.ServiceException;
 import com.openexchange.session.Session;
+import com.openexchange.sessiond.SessionExceptionCodes;
+import com.openexchange.sessiond.SessionMatcher;
 import com.openexchange.sessiond.SessiondEventConstants;
+import com.openexchange.sessiond.SessiondException;
 import com.openexchange.sessiond.cache.SessionCache;
-import com.openexchange.sessiond.exception.SessiondException;
-import com.openexchange.sessiond.exception.SessiondException.Code;
-import com.openexchange.sessiond.services.SessiondServiceRegistry;
 import com.openexchange.threadpool.ThreadPoolService;
 import com.openexchange.timer.ScheduledTimerTask;
 import com.openexchange.timer.TimerService;
 
 /**
  * {@link SessionHandler} - Provides access to sessions
- * 
+ *
  * @author <a href="mailto:sebastian.kauss@open-xchange.com">Sebastian Kauss</a>
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
@@ -85,7 +86,7 @@ public final class SessionHandler {
 
     private static SessionIdGenerator sessionIdGenerator;
 
-    private static SessiondConfigInterface config;
+    static SessiondConfigInterface config;
 
     private static SessionData sessionData;
 
@@ -93,13 +94,15 @@ public final class SessionHandler {
 
     private static final AtomicBoolean initialized = new AtomicBoolean();
 
-    private static final Log LOG = LogFactory.getLog(SessionHandler.class);
+    private static final Log LOG = com.openexchange.log.Log.valueOf(LogFactory.getLog(SessionHandler.class));
 
     private static final boolean INFO = LOG.isInfoEnabled();
 
     private static final boolean DEBUG = LOG.isDebugEnabled();
 
-    private static ScheduledTimerTask sessiondTimer;
+    private static ScheduledTimerTask shortSessionContainerRotator;
+
+    private static ScheduledTimerTask longSessionContainerRotator;
 
     /**
      * Initializes a new {@link SessionHandler session handler}
@@ -110,35 +113,30 @@ public final class SessionHandler {
 
     /**
      * Initializes the {@link SessionHandler session handler}
-     * 
-     * @param config The appropriate configuration
+     *
+     * @param newConfig The appropriate configuration
      */
-    public static void init(final SessiondConfigInterface config) {
-        SessionHandler.config = config;
-        sessionData = new SessionData(config.getNumberOfSessionContainers(), config.getMaxSessions());
+    public static void init(final SessiondConfigInterface newConfig) {
+        SessionHandler.config = newConfig;
+        sessionData = new SessionData(
+            config.getNumberOfSessionContainers(),
+            config.getMaxSessions(),
+            config.getRandomTokenTimeout(),
+            config.getNumberOfLongTermSessionContainers(),
+            config.isAutoLogin());
         if (initialized.compareAndSet(false, true)) {
             try {
                 sessionIdGenerator = SessionIdGenerator.getInstance();
             } catch (final SessiondException exc) {
                 LOG.error("create instance of SessionIdGenerator", exc);
             }
-
-            noLimit = (config.getMaxSessions() == 0);
-
-            final TimerService timer = SessiondServiceRegistry.getServiceRegistry().getService(TimerService.class);
-            if (timer != null) {
-                sessiondTimer =
-                    timer.scheduleWithFixedDelay(
-                        new SessiondTimer(),
-                        config.getSessionContainerTimeout(),
-                        config.getSessionContainerTimeout());
-            }
+            noLimit = (newConfig.getMaxSessions() == 0);
         }
     }
 
     /**
      * Removes all sessions associated with given user in specified context
-     * 
+     *
      * @param userId The user ID
      * @param contextId The context ID
      * @param propagate <code>true</code> for remote removal; otherwise <code>false</code>
@@ -149,8 +147,7 @@ public final class SessionHandler {
         if (propagate) {
             for (final SessionControl sessionControl : retval) {
                 try {
-                    SessionCache.getInstance().putCachedSessionForRemoteRemoval(
-                        ((SessionImpl) sessionControl.getSession()).createCachedSession());
+                    SessionCache.getInstance().putCachedSessionForRemoteRemoval(sessionControl.getSession().createCachedSession());
                 } catch (final CacheException e) {
                     LOG.error("Remote removal failed for session " + sessionControl.getSession().getSecret(), e);
                 } catch (final ServiceException e) {
@@ -160,14 +157,46 @@ public final class SessionHandler {
         }
         if (INFO) {
             LOG.info(new StringBuilder(64).append(propagate ? "Remote" : "Local").append(" removal of user sessions: User=").append(userId).append(
-                ", Context=").append(contextId).toString());
+            ", Context=").append(contextId).toString());
         }
         return retval;
     }
 
     /**
-     * Gets all sessions associated with given user in specified context
+     * Removes all sessions associated with given context.
+     *
+     * @param contextId The context ID
+     * @param propagate <code>true</code> for remote removal; otherwise <code>false</code>
+     */
+    public static void removeContextSessions(final int contextId, final boolean propagate) {
+        final List<SessionControl> list = sessionData.removeContextSessions(contextId);
+        if (propagate) {
+            for (final SessionControl sessionControl : list) {
+                try {
+                    SessionCache.getInstance().putCachedSessionForRemoteRemoval(sessionControl.getSession().createCachedSession());
+                } catch (final AbstractOXException e) {
+                    LOG.warn("Remote removal failed for session " + sessionControl.getSession().getSecret(), e);
+                }
+            }
+        }
+        if (INFO) {
+            LOG.info(new StringBuilder(64).append(propagate ? "Remote" : "Local").append(" removal of sessions: Context=").append(contextId).toString());
+        }
+    }
+
+    /**
+     * Checks for any active session for specified context.
      * 
+     * @param contextId The context identifier
+     * @return <code>true</code> if at least one active session is found; otherwise <code>false</code>
+     */
+    public static boolean hasForContext(final int contextId) {
+        return sessionData.hasForContext(contextId);
+    }
+
+    /**
+     * Gets all sessions associated with given user in specified context
+     *
      * @param userId The user ID
      * @param contextId The context ID
      * @return The wrapper objects for sessions
@@ -176,9 +205,17 @@ public final class SessionHandler {
         return sessionData.getUserSessions(userId, contextId);
     }
 
+    public static SessionControl getAnyActiveSessionForUser(final int userId, final int contextId, final boolean includeLongTerm) {
+        return sessionData.getAnyActiveSessionForUser(userId, contextId, includeLongTerm);
+    }
+
+    public static Session findFirstSessionForUser(final int userId, final int contextId, final SessionMatcher matcher) {
+        return sessionData.findFirstSessionForUser(userId, contextId, matcher);
+    }
+
     /**
      * Adds a new session containing given attributes to session container(s)
-     * 
+     *
      * @param userId The user ID
      * @param loginName The user's login name
      * @param password The user's password
@@ -188,25 +225,20 @@ public final class SessionHandler {
      * @return The session ID associated with newly created session
      * @throws SessiondException If creating a new session fails
      */
-    protected static String addSession(final int userId, final String loginName, final String password, final Context context, final String clientHost, final String login, String authId, String hash) throws SessiondException {
+    protected static String addSession(final int userId, final String loginName, final String password, final Context context, final String clientHost, final String login, final String authId, final String hash, final String client) throws SessiondException {
         checkMaxSessPerUser(userId, context);
+        checkMaxSessPerClient(client, userId, context);
         checkAuthId(login, authId);
         final String sessionId = sessionIdGenerator.createSessionId(loginName, clientHost);
-        final Session session =
+        final SessionImpl session =
             new SessionImpl(userId, loginName, password, context.getContextId(), sessionId, sessionIdGenerator.createSecretId(
                 loginName,
-                Long.toString(System.currentTimeMillis())), sessionIdGenerator.createRandomId(), clientHost, login, authId, hash);
-        /*
-         * Add session
-         */
-        sessionData.addSession(session, config.getLifeTime(), noLimit);
-        /*
-         * Post event for created session
-         */
+                Long.toString(System.currentTimeMillis())), sessionIdGenerator.createRandomId(), clientHost, login, authId, hash, client);
+        // Add session
+        sessionData.addSession(session, noLimit);
+        // Post event for created session
         postSessionCreation(session);
-        /*
-         * Return session ID
-         */
+        // Return session ID
         return sessionId;
     }
 
@@ -215,31 +247,36 @@ public final class SessionHandler {
         if (maxSessPerUser > 0) {
             final int count = sessionData.getNumOfUserSessions(userId, context);
             if (count >= maxSessPerUser) {
-                throw new SessiondException(Code.MAX_SESSION_PER_USER_EXCEPTION, null, I(userId), I(context.getContextId()));
+                throw SessionExceptionCodes.MAX_SESSION_PER_USER_EXCEPTION.create(I(userId), I(context.getContextId()));
             }
         }
     }
 
-    private static void checkAuthId(String login, String authId) throws SessiondException {
+    private static void checkMaxSessPerClient(final String client, final int userId, final Context context) throws SessiondException {
+        if (null == client) {
+            // Nothing to check against
+            return;
+        }
+        final int maxSessPerClient = config.getMaxSessionsPerClient();
+        if (maxSessPerClient > 0) {
+            final int contextId = context.getContextId();
+            final SessionControl[] userSessions = sessionData.getUserSessions(userId, contextId);
+            int cnt = 0;
+            for (final SessionControl sessionControl : userSessions) {
+                if (client.equals(sessionControl.getSession().getClient()) && ++cnt > maxSessPerClient) {
+                    throw SessionExceptionCodes.MAX_SESSION_PER_CLIENT_EXCEPTION.create(client, I(userId), I(contextId));
+                }
+            }
+        }
+    }
+
+    private static void checkAuthId(final String login, final String authId) throws SessiondException {
         sessionData.checkAuthId(login, authId);
     }
 
-    /**
-     * Refreshes the session's last-accessed time stamp
-     * 
-     * @param sessionid The session ID denoting the session
-     * @return <code>true</code> if a refreshing last-accessed time stamp was successful; otherwise <code>false</code>
-     */
-    protected static boolean refreshSession(final String sessionid) {
-        if (DEBUG) {
-            LOG.debug(new StringBuilder("refreshSession <").append(sessionid).append('>').toString());
-        }
-        return sessionData.getSession(sessionid) != null;
-    }
-
     /**
      * Clears the session denoted by given session ID from session container(s)
-     * 
+     *
      * @param sessionid The session ID
      * @return <code>true</code> if a session could be removed; otherwise <code>false</code>
      */
@@ -255,7 +292,7 @@ public final class SessionHandler {
 
     /**
      * Changes the password stored in session denoted by given session ID
-     * 
+     *
      * @param sessionid The session ID
      * @param newPassword The new password
      * @throws SessiondException If changing the password fails
@@ -266,27 +303,41 @@ public final class SessionHandler {
         }
         final SessionControl sessionControl = sessionData.getSession(sessionid);
         if (null == sessionControl) {
-            throw new SessiondException(SessiondException.Code.PASSWORD_UPDATE_FAILED);
+            throw SessionExceptionCodes.PASSWORD_UPDATE_FAILED.create();
         }
         // TODO: Check permission via security service
-        ((SessionImpl) sessionControl.getSession()).setPassword(newPassword);
+        sessionControl.getSession().setPassword(newPassword);
     }
 
-    protected static Session getSessionByRandomToken(final String randomToken, final String localIp) {
-        final SessionControl sessionControl = sessionData.getSessionByRandomToken(randomToken, config.getRandomTokenTimeout(), localIp);
+    protected static Session getSessionByRandomToken(final String randomToken, final String newIP) {
+        final SessionControl sessionControl = sessionData.getSessionByRandomToken(randomToken);
         if (null == sessionControl) {
             return null;
         }
+        /*
+         * Check if local IP should be replaced
+         */
+        if (null != newIP) {
+            /*
+             * Set local IP
+             */
+            final Session session = sessionControl.getSession();
+            final String oldIP = session.getLocalIp();
+            if (!newIP.equals(oldIP)) {
+                LOG.info("Changing IP of session " + session.getSessionID() + " with authID: " + session.getAuthId() + " from " + oldIP + " to " + newIP + '.');
+                session.setLocalIp(newIP);
+            }
+        }
         return sessionControl.getSession();
     }
 
     /**
      * Gets the session associated with given session ID
-     * 
+     *
      * @param sessionId The session ID
      * @return The session associated with given session ID; otherwise <code>null</code> if expired or none found
      */
-    protected static SessionControl getSession(String sessionId) {
+    protected static SessionControl getSession(final String sessionId) {
         if (DEBUG) {
             LOG.debug(new StringBuilder("getSession <").append(sessionId).append('>').toString());
         }
@@ -314,11 +365,45 @@ public final class SessionHandler {
         return sessionControl;
     }
 
+    /**
+     * Gets the session associated with given alternative identifier
+     *
+     * @param alternative identifier The alternative identifier
+     * @return The session associated with given alternative identifier; otherwise <code>null</code> if expired or none found
+     */
+    protected static SessionControl getSessionByAlternativeId(final String altId) {
+        if (DEBUG) {
+            LOG.debug(new StringBuilder("getSessionByAlternativeId <").append(altId).append('>').toString());
+        }
+        final SessionControl sessionControl = sessionData.getSessionByAlternativeId(altId);
+        if (null == sessionControl) {
+            return null;
+        }
+        // Look-up cache if current session wrapped by session-control is marked for removal
+        try {
+            final SessionCache cache = SessionCache.getInstance();
+            final Session session = sessionControl.getSession();
+            final CachedSession cachedSession = cache.getCachedSessionByUser(session.getUserId(), session.getContextId());
+            if (null != cachedSession) {
+                if (cachedSession.isMarkedAsRemoved()) {
+                    cache.removeCachedSession(cachedSession.getSecret());
+                    removeUserSessions(cachedSession.getUserId(), cachedSession.getContextId(), false);
+                    return null;
+                }
+            }
+        } catch (final CacheException e) {
+            LOG.error("Unable to look-up session cache", e);
+        } catch (final ServiceException e) {
+            LOG.error("Unable to look-up session cache", e);
+        }
+        return sessionControl;
+    }
+
     /**
      * Gets (and removes) the session bound to given session identifier in cache.
      * <p>
      * Session is going to be added to local session containers on a cache hit.
-     * 
+     *
      * @param sessionId The session identifier
      * @return A wrapping instance of {@link SessionControl} or <code>null</code>
      */
@@ -334,7 +419,7 @@ public final class SessionHandler {
                 } else {
                     // A cache hit! Add to local session containers
                     LOG.info("Migrate session: " + cachedSession.getSessionId());
-                    return sessionData.addSession(new SessionImpl(cachedSession), config.getLifeTime(), noLimit);
+                    return sessionData.addSession(new SessionImpl(cachedSession), noLimit);
                 }
             }
         } catch (final CacheException e) {
@@ -349,40 +434,46 @@ public final class SessionHandler {
 
     /**
      * Gets all available instances of {@link SessionControl}
-     * 
+     *
      * @return All available instances of {@link SessionControl}
      */
     public static List<SessionControl> getSessions() {
         if (DEBUG) {
             LOG.debug("getSessions");
         }
-        return sessionData.getSessions();
+        return sessionData.getShortTermSessions();
     }
 
     protected static void cleanUp() {
         if (DEBUG) {
             LOG.debug("session cleanup");
         }
-        final List<SessionControl> sessionControls = sessionData.rotate();
+        final List<SessionControl> controls = sessionData.rotateShort();
         if (INFO) {
-            for (final SessionControl sessionControl : sessionControls) {
+            for (final SessionControl sessionControl : controls) {
                 LOG.info("Session timed out. ID: " + sessionControl.getSession().getSessionID());
             }
         }
-        postContainerRemoval(sessionControls);
+        postSessionDataRemoval(controls);
+    }
+
+    protected static void cleanUpLongTerm() {
+        final List<SessionControl> controls = sessionData.rotateLongTerm();
+        if (INFO) {
+            for (final SessionControl control : controls) {
+                LOG.info("Session timed out. ID: " + control.getSession().getSessionID());
+            }
+        }
+        postContainerRemoval(controls);
     }
 
     public static void close() {
         if (initialized.compareAndSet(true, false)) {
-            postContainerRemoval(sessionData.getSessions());
+            postContainerRemoval(sessionData.getShortTermSessions());
             sessionData.clear();
             sessionIdGenerator = null;
             config = null;
             noLimit = false;
-            if (sessiondTimer != null) {
-                sessiondTimer.cancel(true);
-                sessiondTimer = null;
-            }
         }
     }
 
@@ -390,10 +481,18 @@ public final class SessionHandler {
         return sessionData.countSessions();
     }
 
+    public static int[] getNumberOfLongTermSessions() {
+        return sessionData.getLongTermSessionsPerContainer();
+    }
+
+    public static int[] getNumberOfShortTermSessions() {
+        return sessionData.getShortTermSessionsPerContainer();
+    }
+
     private static void postSessionCreation(final Session session) {
         final EventAdmin eventAdmin = getServiceRegistry().getService(EventAdmin.class);
         if (eventAdmin != null) {
-            final Dictionary<Object, Object> dic = new Hashtable<Object, Object>();
+            final Dictionary<String, Object> dic = new Hashtable<String, Object>(1);
             dic.put(SessiondEventConstants.PROP_SESSION, session);
             final Event event = new Event(SessiondEventConstants.TOPIC_ADD_SESSION, dic);
             eventAdmin.postEvent(event);
@@ -406,7 +505,7 @@ public final class SessionHandler {
     private static void postSessionRemoval(final Session session) {
         final EventAdmin eventAdmin = getServiceRegistry().getService(EventAdmin.class);
         if (eventAdmin != null) {
-            final Dictionary<Object, Object> dic = new Hashtable<Object, Object>();
+            final Dictionary<String, Object> dic = new Hashtable<String, Object>(1);
             dic.put(SessiondEventConstants.PROP_SESSION, session);
             final Event event = new Event(SessiondEventConstants.TOPIC_REMOVE_SESSION, dic);
             eventAdmin.postEvent(event);
@@ -419,7 +518,7 @@ public final class SessionHandler {
     private static void postContainerRemoval(final List<SessionControl> sessionControls) {
         final EventAdmin eventAdmin = getServiceRegistry().getService(EventAdmin.class);
         if (eventAdmin != null) {
-            final Dictionary<Object, Object> dic = new Hashtable<Object, Object>();
+            final Dictionary<String, Object> dic = new Hashtable<String, Object>(1);
             final Map<String, Session> eventMap = new HashMap<String, Session>();
             for (final SessionControl sessionControl : sessionControls) {
                 final Session session = sessionControl.getSession();
@@ -434,11 +533,66 @@ public final class SessionHandler {
         }
     }
 
-    public static void addThreadPoolService(ThreadPoolService service) {
+    private static void postSessionDataRemoval(final List<SessionControl> controls) {
+        final EventAdmin eventAdmin = getServiceRegistry().getService(EventAdmin.class);
+        if (eventAdmin != null) {
+            final Dictionary<String, Object> dic = new Hashtable<String, Object>(1);
+            final Map<String, Session> eventMap = new HashMap<String, Session>();
+            for (final SessionControl sessionControl : controls) {
+                final Session session = sessionControl.getSession();
+                eventMap.put(session.getSessionID(), session);
+            }
+            dic.put(SessiondEventConstants.PROP_CONTAINER, eventMap);
+            final Event event = new Event(SessiondEventConstants.TOPIC_REMOVE_DATA, dic);
+            eventAdmin.postEvent(event);
+            if (DEBUG) {
+                LOG.debug("Posted event for removing temporary session data.");
+            }
+        }
+    }
+
+    static void postSessionReactivation(final Session session) {
+        final EventAdmin eventAdmin = getServiceRegistry().getService(EventAdmin.class);
+        if (eventAdmin != null) {
+            final Dictionary<String, Object> dic = new Hashtable<String, Object>(1);
+            dic.put(SessiondEventConstants.PROP_SESSION, session);
+            final Event event = new Event(SessiondEventConstants.TOPIC_ADD_SESSION, dic);
+            eventAdmin.postEvent(event);
+            if (DEBUG) {
+                LOG.debug("Posted event for added session");
+            }
+        }
+    }
+
+    public static void addThreadPoolService(final ThreadPoolService service) {
         sessionData.addThreadPoolService(service);
     }
 
     public static void removeThreadPoolService() {
         sessionData.removeThreadPoolService();
     }
+
+    public static void addTimerService(final TimerService service) {
+        sessionData.addTimerService(service);
+        final long containerTimeout = config.getSessionContainerTimeout();
+        shortSessionContainerRotator =
+            service.scheduleWithFixedDelay(new ShortSessionContainerRotator(), containerTimeout, containerTimeout);
+        if (config.isAutoLogin()) {
+            final long longContainerTimeout = config.getLongTermSessionContainerTimeout();
+            longSessionContainerRotator =
+                service.scheduleWithFixedDelay(new LongSessionContainerRotator(), longContainerTimeout, longContainerTimeout);
+        }
+    }
+
+    public static void removeTimerService() {
+        if (longSessionContainerRotator != null) {
+            longSessionContainerRotator.cancel(false);
+            longSessionContainerRotator = null;
+        }
+        if (shortSessionContainerRotator != null) {
+            shortSessionContainerRotator.cancel(false);
+            shortSessionContainerRotator = null;
+        }
+        sessionData.removeTimerService();
+    }
 }
