@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,12 +49,13 @@
 
 package com.openexchange.sessiond.impl;
 
-import com.openexchange.sessiond.exception.SessiondException;
+import com.openexchange.sessiond.SessionExceptionCodes;
+import com.openexchange.sessiond.SessiondException;
 
 /**
  * {@link SessionIdGenerator} - The session ID generator
- * 
- * @author <a href="mailto:sebastian.kauss@netline-is.de">Sebastian Kauss</a>
+ *
+ * @author <a href="mailto:sebastian.kauss@open-xchange.com">Sebastian Kauss</a>
  */
 
 public abstract class SessionIdGenerator {
@@ -75,7 +76,7 @@ public abstract class SessionIdGenerator {
 
     /**
      * Creates a new instance implementing the group storage interface.
-     * 
+     *
      * @param context Context.
      * @return an instance implementing the group storage interface.
      * @throws LdapException if the instance can't be created.
@@ -84,13 +85,13 @@ public abstract class SessionIdGenerator {
         try {
             return getImplementingClass().getConstructor().newInstance();
         } catch (final Exception exc) {
-            throw new SessiondException(SessiondException.Code.SESSIOND_EXCEPTION, exc, new Object[0]);
+            throw SessionExceptionCodes.SESSIOND_EXCEPTION.create(exc);
         }
     }
 
     /**
      * Proxy method to get the implementing class.
-     * 
+     *
      * @return the class implementing this interface.
      * @throws ClassNotFoundException if the class can't be loaded.
      */
