@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -50,20 +50,31 @@
 package com.openexchange.sessiond.impl;
 
 import java.io.Serializable;
-import java.util.Iterator;
 import java.util.Map;
+import java.util.Map.Entry;
 import java.util.concurrent.ConcurrentHashMap;
+import java.util.concurrent.atomic.AtomicInteger;
+import java.util.concurrent.locks.ReentrantLock;
+import org.apache.commons.logging.Log;
+import org.apache.commons.logging.LogFactory;
 import com.openexchange.caching.objects.CachedSession;
+import com.openexchange.config.ConfigurationService;
+import com.openexchange.crypto.CryptoException;
+import com.openexchange.crypto.CryptoService;
 import com.openexchange.session.Session;
+import com.openexchange.sessiond.services.SessiondServiceRegistry;
 
 /**
  * {@link SessionImpl} - Implements interface {@link Session}
- * 
+ *
  * @author <a href="mailto:sebastian.kauss@open-xchange.com">Sebastian Kauss</a>
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public final class SessionImpl implements Session {
 
+    // A random-enough key for encrypting and decrypting passwords on their way through the caching system.
+    private static final String OBFUSCATION_KEY_PROPERTY = "com.openexchange.sessiond.encryptionKey";
+
     private final String loginName;
 
     private String password;
@@ -85,12 +96,16 @@ public final class SessionImpl implements Session {
     private final String authId;
 
     private String hash;
-    
+
+    private String client;
+
     private final Map<String, Object> parameters;
 
+    private static final Log LOG = com.openexchange.log.Log.valueOf(LogFactory.getLog(SessionImpl.class));
+
     /**
      * Initializes a new {@link SessionImpl}
-     * 
+     *
      * @param userId The user ID
      * @param loginName The login name
      * @param password The password
@@ -100,7 +115,7 @@ public final class SessionImpl implements Session {
      * @param randomToken The random token
      * @param localIp The local IP
      */
-    public SessionImpl(final int userId, final String loginName, final String password, final int contextId, final String sessionId, final String secret, final String randomToken, final String localIp, final String login, String authId, String hash) {
+    public SessionImpl(final int userId, final String loginName, final String password, final int contextId, final String sessionId, final String secret, final String randomToken, final String localIp, final String login, final String authId, final String hash, final String client) {
         this.userId = userId;
         this.loginName = loginName;
         this.password = password;
@@ -112,12 +127,16 @@ public final class SessionImpl implements Session {
         this.login = login;
         this.authId = authId;
         this.hash = hash;
+        this.client = client;
         parameters = new ConcurrentHashMap<String, Object>();
+        parameters.put(PARAM_LOCK, new ReentrantLock());
+        parameters.put(PARAM_COUNTER, new AtomicInteger());
+        parameters.put(PARAM_ALTERNATIVE_ID, UUIDSessionIdGenerator.randomUUID());
     }
 
     /**
      * Initializes a new {@link SessionImpl} from specified cached session.
-     * 
+     *
      * @param cachedSession The cached session
      */
     protected SessionImpl(final CachedSession cachedSession) {
@@ -125,7 +144,7 @@ public final class SessionImpl implements Session {
         userId = cachedSession.getUserId();
         contextId = cachedSession.getContextId();
         loginName = cachedSession.getLoginName();
-        password = cachedSession.getPassword();
+        password = unobfuscate( cachedSession.getPassword() );
         sessionId = cachedSession.getSessionId();
         secret = cachedSession.getSecret();
         randomToken = cachedSession.getRandomToken();
@@ -133,21 +152,62 @@ public final class SessionImpl implements Session {
         localIp = cachedSession.getLocalIp();
         authId = cachedSession.getAuthId();
         hash = cachedSession.getHash();
+        client = cachedSession.getClient();
         final Map<String, Serializable> params = cachedSession.getParameters();
         parameters = new ConcurrentHashMap<String, Object>(params.size());
-        for (final Iterator<Map.Entry<String, Serializable>> iter = params.entrySet().iterator(); iter.hasNext();) {
-            final Map.Entry<String, Serializable> entry = iter.next();
+        for (final Entry<String, Serializable> entry : params.entrySet()) {
             parameters.put(entry.getKey(), entry.getValue());
         }
+        parameters.put(PARAM_LOCK, new ReentrantLock());
+        parameters.put(PARAM_COUNTER, new AtomicInteger());
+        if (!parameters.containsKey(PARAM_ALTERNATIVE_ID)) {
+            parameters.put(PARAM_ALTERNATIVE_ID, UUIDSessionIdGenerator.randomUUID());
+        }
     }
 
     /**
      * Creates a new instance of {@link CachedSession} holding this session's state and information ready for being put into session cache.
-     * 
+     *
      * @return An appropriate instance of {@link CachedSession}
      */
     public CachedSession createCachedSession() {
-        return new CachedSession(userId, loginName, password, contextId, sessionId, secret, randomToken, localIp, login, authId, hash, parameters);
+        return new CachedSession(userId, loginName, obfuscate( password ), contextId, sessionId, secret, randomToken, localIp, login, authId, hash, client, parameters);
+    }
+
+    private String obfuscate(final String string) {
+        try {
+            final String key = getObfuscationKey();
+            return isEmpty(key) ? string : SessiondServiceRegistry.getServiceRegistry().getService(CryptoService.class).encrypt(string, key);
+        } catch (final CryptoException e) {
+            LOG.error("Could not obfuscate a string before migration", e);
+            return string;
+        }
+    }
+
+    private static boolean isEmpty(final String str) {
+        if (null == str) {
+            return true;
+        }
+        final int length = str.length();
+        boolean empty = true;
+        for (int i = 0; empty && i < length; i++) {
+            empty = Character.isWhitespace(str.charAt(i));
+        }
+        return empty;
+    }
+
+    private String getObfuscationKey() {
+        return SessiondServiceRegistry.getServiceRegistry().getService(ConfigurationService.class).getProperty(OBFUSCATION_KEY_PROPERTY);
+    }
+
+    private String unobfuscate(final String string) {
+        try {
+            final String key = getObfuscationKey();
+            return SessiondServiceRegistry.getServiceRegistry().getService(CryptoService.class).decrypt(string, key);
+        } catch (final CryptoException e) {
+            LOG.error("Could not decode string after migration", e);
+            return string;
+        }
     }
 
     public int getContextId() {
@@ -179,6 +239,9 @@ public final class SessionImpl implements Session {
     }
 
     public void setParameter(final String name, final Object value) {
+        if (PARAM_LOCK.equals(name)) {
+            return;
+        }
         if (null == value) {
             parameters.remove(name);
         } else {
@@ -193,7 +256,7 @@ public final class SessionImpl implements Session {
     public String getLocalIp() {
         return localIp;
     }
-    
+
 
     public void setLocalIp(final String localIp) {
         this.localIp = localIp;
@@ -221,7 +284,7 @@ public final class SessionImpl implements Session {
 
     /**
      * Sets the password
-     * 
+     *
      * @param password The password to set
      */
     public void setPassword(final String password) {
@@ -235,8 +298,16 @@ public final class SessionImpl implements Session {
     public String getHash() {
         return hash;
     }
-    
-    public void setHash(String hash) {
+
+    public void setHash(final String hash) {
         this.hash = hash;
     }
+
+    public String getClient() {
+        return client;
+    }
+
+    public void setClient(final String client) {
+        this.client = client;
+    }
 }
