@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -51,43 +51,33 @@ package com.openexchange.sessiond.impl;
 
 /**
  * SessionConfig
- * 
+ *
  * @author <a href="mailto:sebastian.kauss@open-xchange.org">Sebastian Kauss</a>
  */
-
 public interface SessiondConfigInterface {
 
-    public int getServerPort();
+    long getSessionContainerTimeout();
 
-    public int getServerObjectStreamPort();
+    long getLongTermSessionContainerTimeout();
 
-    public boolean isServerSocketEnabled();
+    long getNumberOfSessionContainers();
 
-    public boolean isServerObjectStreamSocketEnabled();
+    int getMaxSessions();
 
-    public boolean isTcpClientSocketEnabled();
+    int getMaxSessionsPerUser();
 
-    public int getSessionContainerTimeout();
+    int getMaxSessionsPerClient();
 
-    public int getNumberOfSessionContainers();
+    long getLifeTime();
 
-    public int getMaxSessions();
+    long getLongLifeTime();
 
-    public int getMaxSessionsPerUser();
+    long getRandomTokenTimeout();
 
-    public String getServerBindAddress();
+    long getNumberOfLongTermSessionContainers();
 
-    public boolean isDoubleLoginPermitted();
-
-    public String getSessionAuthUser();
-
-    public int getLifeTime();
-
-    public int getRandomTokenTimeout();
-
-    public String getCAFile();
-
-    public String getCertFile();
-
-    public String getKeyFile();
+    /**
+     * @return <code>true</code> if autologin is enabled.
+     */
+    boolean isAutoLogin();
 }
