@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -56,32 +56,26 @@ import org.apache.commons.logging.LogFactory;
 import com.openexchange.config.ConfigurationService;
 import com.openexchange.groupware.AbstractOXException;
 import com.openexchange.server.Initialization;
+import com.openexchange.sessiond.SessionExceptionCodes;
 import com.openexchange.sessiond.cache.SessionCache;
 import com.openexchange.sessiond.cache.SessionCacheConfiguration;
-import com.openexchange.sessiond.cache.SessionCacheTimer;
-import com.openexchange.sessiond.exception.SessiondException;
-import com.openexchange.sessiond.services.SessiondServiceRegistry;
-import com.openexchange.timer.ScheduledTimerTask;
-import com.openexchange.timer.TimerService;
 
 /**
  * {@link SessiondInit} - Initializes sessiond service
- * 
- * @author <a href="mailto:sebastian.kauss@netline-is.de">Sebastian Kauss</a>
+ *
+ * @author <a href="mailto:sebastian.kauss@open-xchange.com">Sebastian Kauss</a>
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public class SessiondInit implements Initialization {
 
-    private static final Log LOG = LogFactory.getLog(SessiondInit.class);
+    private static final Log LOG = com.openexchange.log.Log.valueOf(LogFactory.getLog(SessiondInit.class));
 
-    private SessiondConfigImpl config;
+    private SessiondConfigInterface config;
 
     private final AtomicBoolean started = new AtomicBoolean();
 
     private static final SessiondInit singleton = new SessiondInit();
 
-    private ScheduledTimerTask sessionCacheTimer;
-
     public static SessiondInit getInstance() {
         return singleton;
     }
@@ -107,25 +101,18 @@ public class SessiondInit implements Initialization {
                 sessiond.start();
                 started.set(true);
             } else {
-                throw new SessiondException(SessiondException.Code.SESSIOND_CONFIG_EXCEPTION);
+                throw SessionExceptionCodes.SESSIOND_CONFIG_EXCEPTION.create();
             }
 
             SessionCacheConfiguration.getInstance().start();
-            
-            final TimerService timer = SessiondServiceRegistry.getServiceRegistry().getService(TimerService.class, true);
-            sessionCacheTimer = timer.scheduleWithFixedDelay(new SessionCacheTimer(), 0, 30000);
         }
     }
 
-    public void stop() throws AbstractOXException {
+    public void stop() {
         if (!started.get()) {
             LOG.error(SessiondInit.class.getName() + " has not been started");
             return;
         }
-        if (null != sessionCacheTimer) {
-            sessionCacheTimer.cancel(false);
-            sessionCacheTimer = null;
-        }
         SessionCacheConfiguration.getInstance().stop();
         SessionCache.releaseInstance();
         final Sessiond s = Sessiond.getInstance(config);
@@ -135,7 +122,7 @@ public class SessiondInit implements Initialization {
 
     /**
      * Checks if {@link SessiondInit} is started
-     * 
+     *
      * @return <code>true</code> if {@link SessiondInit} is started; otherwise <code>false</code>
      */
     public boolean isStarted() {
