@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -55,16 +55,14 @@ import com.openexchange.sessiond.SessiondMBean;
 
 /**
  * {@link SessiondMBeanImpl}
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public final class SessiondMBeanImpl extends StandardMBean implements SessiondMBean {
 
-    private static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(SessiondMBeanImpl.class);
-
     /**
      * Initializes a new {@link SessiondMBeanImpl}
-     * 
+     *
      * @throws NotCompliantMBeanException If the mbeanInterface does not follow JMX design patterns for Management Interfaces, or if this
      *             does not implement the specified interface.
      */
@@ -76,4 +74,31 @@ public final class SessiondMBeanImpl extends StandardMBean implements SessiondMB
         return SessionHandler.removeUserSessions(userId, contextId, true).length;
     }
 
+    public void clearContextSessions(final int contextId) {
+        /*-
+         * TODO:
+         * Put invalidated-marker into session cache
+        try {
+            final Cache sessionCache = SessionCache.getInstance().getCache();
+            final Integer key = Integer.valueOf(contextId);
+            sessionCache.put(key, InvalidatedMarker.newInstance(key));
+        } catch (final AbstractOXException e) {
+            // Ignore
+        }
+         *
+         */
+        /*
+         * Clear context-associated sessions
+         */
+        SessionHandler.removeContextSessions(contextId, true);
+    }
+
+    public int[] getNumberOfShortTermSessions() {
+        return SessionHandler.getNumberOfShortTermSessions();
+    }
+
+    public int[] getNumberOfLongTermSessions() {
+        return SessionHandler.getNumberOfLongTermSessions();
+    }
+
 }
